/* ok
 * Rezolvare OFFLINE
 * Mai intai se plaseaza in arbore valorile v = 1, apoi se raspunde la toate intrebarile care au k = 1
 * Apoi se plaseaza in arbore valorile v = 2 si urmeaza intrebarile pt k = 2, etc.
 * Practic, la update se plaseaza 1 pt fiecare nod x cu valoare in limita curenta k
 * si la query-ul curent se numara valorile de 1 existente pe lantul x-y
 */
#include <cstdio>
#include <cstring>
#include <algorithm>
#include <vector>
#include <vector>
#include <cctype>
#include <cmath>
using namespace std;

FILE *fin = fopen("viteza.in","r");
FILE *fout = fopen("viteza.out","w");

const int LOG = 17;
const int NMAX = 1e5;
const int MMAX = 1e5;

struct Event {
	int x, y, k;
	int id;

    // sortam crescator dupa valoarea k si la valori egale, update-urile (adaugarea de valori in noduri) vin inaintea Event-urilor
	bool operator < (const Event &q) const
	{
		if (k < q.k || (k == q.k && y < q.y))    // daca e update pe nod, y = 0
			return true;
		return false;
	}

	bool IsQuery()
	{
		return y != 0;
	}
};

int n,m;
vector<Event> events;
vector<int> G[NMAX + 1];

int tin[NMAX + 1];       // timpii de descoperire ai nodurilor
int tout[NMAX + 1];      // timpii de terminare
int timer;
int t[LOG + 1][NMAX + 1]; // stramosii
int niv[NMAX + 1];

int aib[NMAX + 5];
int ans[NMAX + 5]; // ans[i] = raspunsul la intrebarea i
int v[NMAX + 5];   // valorile asociate nodurilor

void Update(int pos, int val);  // aib - update
// Sum(tin[x]) returneaza numarul de noduri pe lantul de la radacina la nodul x, care au costul mai mic sau egal cu k din query-ul curent
int QuerySum(int pos);               // aib - query
void Dfs(int x, int tata);
int LCA(int x, int y);
int GetInt();                   // parsarea intrarii

int main()
{
    n = GetInt();
    m = GetInt();
	int x, y;
    for (int i = 1; i < n; ++i)
    {
        x = GetInt();
        y = GetInt();
        G[x].push_back(y);
        G[y].push_back(x);
    }

    Dfs(1, 0);

    for (int i = 1; (1 << i) <= n; ++i)
        for(int j = 1; j <= n; ++j)
            t[i][j] = t[i - 1][t[i - 1][j]];

    for (int x = 1; x <= n; ++x)
    {
        v[x] = GetInt();
        events.emplace_back(x, 0, v[x], 0);  // eveniment de updata
    }

    for (int i = 1, k; i <= m; ++i)
    {
        x = GetInt();
        y = GetInt();
        k = GetInt();
        events.emplace_back(x, y, k, i);
    }

    sort(events.begin(),events.end());

	int lca = 0;
    for (auto q : events)
        if (q.IsQuery())
        {
            x = q.x;
            y = q.y;
            lca = LCA(x, y);
            ans[q.id] = QuerySum(tin[x]) + QuerySum(tin[y]) - 2 * QuerySum(tin[lca]) + (v[lca] <= q.k);
        }
        else
        {
			// update pt toate nodurile cu valoarea k urmatoare
            Update(tin[q.x], 1);
            Update(tout[q.x] + 1, -1);
        }

    for (int i = 1; i <= m; ++i)
        fprintf(fout, "%d\n", ans[i]);

    fclose(fin);
    fclose(fout);

    return 0;
}

void Dfs(int x, int father)
{
    niv[x] = niv[father] + 1;
    t[0][x] = father;
    tin[x] = ++timer;

    for (int const& y : G[x])
    {
        if (y == father)  continue;
        Dfs(y, x);
    }

    tout[x] = timer;
}

// Fenwick tree
void Update(int pos, int val)
{
    for (int i = pos; i <= n; i += i & -i)
        aib[i] += val;
}

int QuerySum(int pos)
{
    int sum = 0;
    for (int i = pos; i; i -=  i & -i)
        sum += aib[i];

    return sum;
}

inline int LCA(int x, int y)
{
	// x trebuie sa aiba nivelul cel mai mare
	if (niv[x] < niv[y])
		swap(x, y);

    // aducem la acelasi nivel x si y
    int dif = niv[x] - niv[y];
    for (int i = 0; (1 << i) <= dif; ++i)
		if (dif & (1 << i))
			x = t[i][x];

    if (x == y) return x;

    for (int i = log2(niv[x]); i >= 0; --i) // sarim (jump) cu x si y cu puteri ale lui 2 pana pe cei doi fii ai lui LCA
        if (t[i][x] != t[i][y])
        {
            x = t[i][x];
            y = t[i][y];
        }
    return t[0][x];   // tata lui x este LCA
}

/// parsarea intrarii
const int LIM = 1 << 17;
char buff[LIM];
int ind = LIM - 1;

int GetInt()
{
    int nr = 0;
    while (!isdigit(buff[ind]))
        if (++ind >= LIM)
        {
            ind = 0;
            fread(buff, 1, LIM, fin);
        }

    while (isdigit(buff[ind]))
    {
        nr = nr * 10 + (buff[ind] - '0');
        if (++ind >= LIM)
        {
            ind = 0;
            fread(buff, 1, LIM, fin);
        }
    }
    return nr;
}
