/**     Tree decomposition
    Liniarizarea arborelui - se aplica NUMAI pentru SUME

	Suma lantului de la x la y = suma(x, LCA) + suma(LCA, y) - val[LCA]

	Se da un arbore cu valori in noduri.
	Operatiile:
	0 x y  (update: ADAUGA valoarea y la nodul x)
	1 x y  (query:  determina suma lantului de la x la y)

 */
#include <fstream>
#include <vector>
#include <cmath>
#include <iostream>
using namespace std;

ifstream fin("arb.in");
ofstream fout("arb.out");

typedef vector<int> VI;

const int DIM = 10005;
int aint[4 * DIM];             // tine sume pe intervale pentru sirul a
//int a[DIM];                  // nu este necesar (-sirul pentru care se pastreaza sumele pe intervale in "aint")
int tin[DIM], tout[DIM];       // tin[x] / tout[x] - poz primei aparitii/ultimei a lui x in sirul "a"
int timer;                     // poz in sirul a[]

// LCA - RMQ
int E[2 * DIM], pozE[2 * DIM]; // pozE[x]   - poz primei aparitii a nodului x in turul Euler
int niv[2 * DIM];              // N         - poz in turul Euler si in niv
int p[17][DIM];                // p[i][j]   - pozitia minimului in sirul niv[], incepand de la poz j si de lungime 2^i
int N;
bool viz[DIM];
VI G[DIM];
int val[2 * DIM];   // valorile din noduri
int n, m;

void Read();
void Update(int x, int l, int r, int p, int v);
int Query(int x, int l, int r, int a, int b);
void Dfs(int x, int nv);
void RMQ(); // RMQ
int LCA(int x, int y);

int main()
{
    Read();
    Dfs(1, 1);
    RMQ();
//	Debug();

    fin >> m;
	int op, x, y;
    for (int i = 1; i <= m; ++i)
    {
        fin >> op >> x >> y; // op = 0 update, op = 1, query (suma pe lantul de la x la y)
        if (op == 0)
        {
            val[x] = y;
            Update(1, 1, n + 1, tin[x], y);            // adunam si scadem valoarea noua
            Update(1, 1, n + 1, tout[x] + 1, -y);
        }
        else
        {
			int lca = LCA(x, y);
          // OK
            int s1 = Query(1, 1, n + 1, tin[lca], tin[x]);
            int s2 = Query(1, 1, n + 1, tin[lca], tin[y]);

            fout << s1 + s2 - val[lca] << '\n';
            // sau
            // fout << s1 + s2 - Query(1, 1, n + 1, tin[lca], tin[lca]) << '\n';
        }
    }

    return 0;
}

void Dfs(int x, int nv)
{
    viz[x] = true;
    tin[x] = ++timer;
    // la momentul descoperirii nodului x, introducem valoarea v in arb de intervale (se putea face si ulterior)
    Update(1, 1, n + 1, tin[x], val[x]);

    E[++N] = x;
    pozE[x] = N;  // prima aparitie a lui x in turul Euler
    niv[N] = nv;

    for (int y : G[x])
        if (!viz[y])
		{
			Dfs(y, nv + 1);
			E[++N] = x;
			niv[N] = nv;
		}

    tout[x] = timer;  // nu se mai incrementeaza timpul!
    Update(1, 1, n + 1, tout[x] + 1, -val[x]);  // ATENTIE!  tout[x] + 1
}

void Update(int x, int l, int r, int p, int v)
{
    if (l == r)
    {
        aint[x] += v;   // OBLIGATORIU +=   (NU Merge aint[x] = v)
        return;
    }

    int m = (l + r) >> 1;
    if (p <= m)
        Update(2 * x, l, m, p, v);
    else
        Update(2 * x + 1, m + 1, r, p, v);
    aint[x] = aint[2 * x] + aint[2 * x + 1];
}

int Query(int x, int l, int r, int a, int b)
{
    if (a <= l and r <= b)
		return aint[x];

    int m = (l + r) >> 1;
    int sum = 0;
    if (a <= m)
        sum = Query(2 * x, l, m, a, b);
    if (b > m)
        sum += Query(2 * x + 1, m + 1, r, a, b);
    return sum;
}

int LCA(int x, int y)
{
	int px = pozE[x];    // px = pozitia lui x in sirul niv[] (in turul Euler)
	int py = pozE[y];
	if (px > py)
		swap(px, py);

	int k = log2(py - px + 1);
	int p1 = p[k][px];
	int p2 = p[k][py - (1 << k) + 1];

	if (niv[p1] < niv[p2])
		return E[p1];

	return E[p2];
}

void RMQ() // O(N * log2(N))
{
	int N = 2 * n;
	for (int i = 1; i <= N; ++i)
		p[0][i] = i;

	int p1, p2;
    for (int i = 1; (1 << i) <= N; ++i)
        for (int j = 1; j + (1 << i) <= N; ++j)
        {
			p1 = p[i - 1][j];
			p2 = p[i - 1][j + (1 << (i - 1))];

            if (niv[p1] < niv[p2])
                p[i][j] = p1;
            else
                p[i][j] = p2;
		}
}

void Read()
{
    fin >> n;
    for (int i = 1; i <= n; ++i)
        fin >> val[i];
    int x, y;
    for (int i = 1; i < n; ++i)
    {
        fin >> x >> y;
        G[x].push_back(y);
        G[y].push_back(x);
    }
}

void Debug()
{
    cout << "tin: ";
    for ( int i = 1; i <= n; ++i )
        fout << tin[i] << ' ';
    cout << "\n " ;

    cout << "tout: ";
    for ( int i = 1; i <= n; ++i )
        fout << tout[i] << ' ';
    cout << "\n " ;
/*
    for ( int i = 1; i <= n; ++i )
        cout << a[i] << ' ';
    cout << "\n";
*/
	cout << "E: ";
    for ( int i = 1; i <= N; ++i )
        cout << E[i] << ' ';
    cout << "\nniv: ";
    for ( int i = 1; i <= N; ++i )
        cout << niv[i] << ' ';
    cin.get();
}
